@@ -154,7 +154,7 @@ function generate_initializesystem_timevarying(sys::AbstractSystem;
     # 3) process other variables
     for var in vars
         if var ∈ keys(op)
-            push!(eqs_ics, var ~ defs[var])
+            push!(eqs_ics, var ~ op[var])
         elseif var ∈ keys(guesses)
             push!(defs, var => guesses[var])
         elseif check_defguess
@@ -669,7 +669,7 @@ function SciMLBase.remake_initialization_data(
 end
 
 promote_type_with_nothing(::Type{T}, ::Nothing) where {T} = T
-promote_type_with_nothing(::Type{T}, ::SizedVector{0}) where {T} = T
+promote_type_with_nothing(::Type{T}, ::StaticVector{0}) where {T} = T
 function promote_type_with_nothing(::Type{T}, ::AbstractArray{T2}) where {T, T2}
     promote_type(T, T2)
 end
@@ -678,7 +678,7 @@ function promote_type_with_nothing(::Type{T}, p::MTKParameters) where {T}
 end
 
 promote_with_nothing(::Type, ::Nothing) = nothing
-promote_with_nothing(::Type, x::SizedVector{0}) = x
+promote_with_nothing(::Type, x::StaticVector{0}) = x
 promote_with_nothing(::Type{T}, x::AbstractArray{T}) where {T} = x
 function promote_with_nothing(::Type{T}, x::AbstractArray{T2}) where {T, T2}
     if ArrayInterface.ismutable(x)
@@ -824,8 +824,6 @@ Counteracts the CSE/array variable hacks in `symbolics_tearing.jl` so it works w
 initialization.
 """
 function unhack_observed(obseqs::Vector{Equation}, eqs::Vector{Equation})
-    subs = Dict()
-    tempvars = Set()
     rm_idxs = Int[]
     for (i, eq) in enumerate(obseqs)
         iscall(eq.rhs) || continue
@@ -835,20 +833,7 @@ function unhack_observed(obseqs::Vector{Equation}, eqs::Vector{Equation})
         end
     end
 
-    for (i, eq) in enumerate(obseqs)
-        if eq.lhs in tempvars
-            subs[eq.lhs] = eq.rhs
-            push!(rm_idxs, i)
-        end
-    end
-
     obseqs = obseqs[setdiff(eachindex(obseqs), rm_idxs)]
-    obseqs = map(obseqs) do eq
-        fixpoint_sub(eq.lhs, subs) ~ fixpoint_sub(eq.rhs, subs)
-    end
-    eqs = map(eqs) do eq
-        fixpoint_sub(eq.lhs, subs) ~ fixpoint_sub(eq.rhs, subs)
-    end
     return obseqs, eqs
 end
 
