@@ -1,6 +1,3 @@
-abstract type AbstractDynamicOptProblem{uType, tType, isinplace} <:
-              SciMLBase.AbstractODEProblem{uType, tType, isinplace} end
-
 abstract type AbstractCollocation end
 
 struct DynamicOptSolution
@@ -21,9 +18,9 @@ end
 """
     JuMPDynamicOptProblem(sys::System, op, tspan; dt, steps, guesses, kwargs...)
 
-Convert an System representing an optimal control system into a JuMP model
-for solving using optimization. Must provide either `dt`, the timestep between collocation 
-points (which, along with the timespan, determines the number of points), or directly 
+Convert a System representing an optimal control system into a JuMP model
+for solving using optimization. Must provide either `dt`, the timestep between collocation
+points (which, along with the timespan, determines the number of points), or directly
 provide the number of points as `steps`.
 
 To construct the problem, please load InfiniteOpt along with ModelingToolkit.
@@ -32,8 +29,8 @@ function JuMPDynamicOptProblem end
 """
     InfiniteOptDynamicOptProblem(sys::System, op, tspan; dt)
 
-Convert an System representing an optimal control system into a InfiniteOpt model
-for solving using optimization. Must provide `dt` for determining the length 
+Convert a System representing an optimal control system into a InfiniteOpt model
+for solving using optimization. Must provide `dt` for determining the length
 of the interpolation arrays.
 
 Related to `JuMPDynamicOptProblem`, but directly adds the differential equations
@@ -45,9 +42,9 @@ function InfiniteOptDynamicOptProblem end
 """
     CasADiDynamicOptProblem(sys::System, op, tspan; dt, steps, guesses, kwargs...)
 
-Convert an System representing an optimal control system into a CasADi model
-for solving using optimization. Must provide either `dt`, the timestep between collocation 
-points (which, along with the timespan, determines the number of points), or directly 
+Convert a System representing an optimal control system into a CasADi model
+for solving using optimization. Must provide either `dt`, the timestep between collocation
+points (which, along with the timespan, determines the number of points), or directly
 provide the number of points as `steps`.
 
 To construct the problem, please load CasADi along with ModelingToolkit.
@@ -56,9 +53,9 @@ function CasADiDynamicOptProblem end
 """
     PyomoDynamicOptProblem(sys::System, op, tspan; dt, steps)
 
-Convert an System representing an optimal control system into a Pyomo model
-for solving using optimization. Must provide either `dt`, the timestep between collocation 
-points (which, along with the timespan, determines the number of points), or directly 
+Convert a System representing an optimal control system into a Pyomo model
+for solving using optimization. Must provide either `dt`, the timestep between collocation
+points (which, along with the timespan, determines the number of points), or directly
 provide the number of points as `steps`.
 
 To construct the problem, please load Pyomo along with ModelingToolkit.
@@ -225,17 +222,69 @@ function process_tspan(tspan, dt, steps)
     end
 end
 
+function get_discrete_time_evaluations(expr)
+    vars = Symbolics.get_variables(expr)
+
+    # Group by base variable
+    result = Dict()
+
+    for v in vars
+        if iscall(v)
+            args = arguments(v)
+            if length(args) == 1 && args[1] isa Number
+                base_var = operation(v)
+                time_point = args[1]
+
+                if !haskey(result, base_var)
+                    result[base_var] = Float64[]
+                end
+                push!(result[base_var], time_point)
+            end
+        end
+    end
+
+    # Sort and unique the time points
+    for (var, times) in result
+        result[var] = sort!(unique!(times))
+    end
+
+    return result
+end
+
+function check_collocation_time_mismatch(sys, expected_tsteps, tsteps)
+    if collect(expected_tsteps)≠tsteps
+        eval_times = get_discrete_time_evaluations(cost(sys))
+        for (var, ts) in eval_times
+            tdiff = setdiff(ts, expected_tsteps)
+            @info tdiff
+            if !isempty(tdiff)
+                error("$var is evaluated inside the cost function at $(length(tdiff)) points " *
+                    "that are not in the $(length(expected_tsteps)) collocation points. " *
+                    "Cost evaluation points must align with the collocation grid. "*
+                    "Adjust the dt to match the time points used in the cost function.")
+            end
+        end
+        if length(expected_tsteps) ≠ length(tsteps)
+            error("Found extra $(abs(length(expected_tsteps) - length(tsteps))) collocation points.")
+        elseif maximum(abs.(expected_tsteps .- tsteps)) > 1e-10
+            error("The collocation points differ from the expected ones by more than 1e-10.")
+        end
+    end
+end
+
 ##########################
 ### MODEL CONSTRUCTION ###
 ##########################
 function process_DynamicOptProblem(
-        prob_type::Type{<:AbstractDynamicOptProblem}, model_type, sys::System, op, tspan;
+        prob_type::Type{<:SciMLBase.AbstractDynamicOptProblem}, model_type, sys::System, op, tspan;
         dt = nothing,
         steps = nothing,
+        tune_parameters = false,
         guesses = Dict(), kwargs...)
     warn_overdetermined(sys, op)
     ctrls = unbound_inputs(sys)
     states = unknowns(sys)
+    tunable_params = tune_parameters ? tunable_parameters(sys) : []
 
     stidxmap = Dict([v => i for (i, v) in enumerate(states)])
     op = Dict([default_toterm(value(k)) => v for (k, v) in op])
@@ -249,18 +298,35 @@ function process_DynamicOptProblem(
     model_tspan, steps, is_free_t = process_tspan(tspan, dt, steps)
     warn_overdetermined(sys, op)
 
-    pmap = filter(p -> (first(p) ∉ Set(unknowns(sys))), op)
-    pmap = recursive_unwrap(AnyDict(pmap))
-    evaluate_varmap!(pmap, keys(pmap))
+    # Build pmap for symbolic substitution in costs/constraints/bounds
+    all_parameters = default_toterm.(parameters(sys))
+    # Extract all parameter values from processed p (which has defaults filled in)
+    getter = SymbolicIndexingInterface.getp(sys, all_parameters)
+    pmap = AnyDict(all_parameters .=> getter(p))
+
+    # Filter out tunable parameters - they should remain symbolic
+    tunable_set = Set(default_toterm.(tunable_params))
+    pmap = filter(kvp -> first(kvp) ∉ tunable_set, pmap)
+
     c0 = value.([pmap[c] for c in ctrls])
+    p0, _ = SciMLStructures.canonicalize(SciMLStructures.Tunable(), p)
 
     tsteps = LinRange(model_tspan[1], model_tspan[2], steps)
     model = generate_internal_model(model_type)
     generate_time_variable!(model, model_tspan, tsteps)
     U = generate_state_variable!(model, u0, length(states), tsteps)
     V = generate_input_variable!(model, c0, length(ctrls), tsteps)
+    P = generate_tunable_params!(model, p0, length(tunable_params))
+    # Add the symbolic representation of the tunable parameters to the map
+    # The order of the Tunable section is given by the tunable_parameters(sys)
+    # Some backends need symbolic accessors instead of raw variables
+    P_syms = [get_param_for_pmap(model, P, i) for i in eachindex(tunable_params)]
+    P_backend = needs_individual_tunables(model) ? P_syms : P
+
     tₛ = generate_timescale!(model, get(pmap, tspan[2], tspan[2]), is_free_t)
-    fullmodel = model_type(model, U, V, tₛ, is_free_t)
+    fullmodel = model_type(model, U, V, P_backend, tₛ, is_free_t, tsteps)
+
+    merge!(pmap, Dict(tunable_params .=> P_syms))
 
     set_variable_bounds!(fullmodel, sys, pmap, tspan[2])
     add_cost_function!(fullmodel, sys, tspan, pmap)
@@ -274,9 +340,28 @@ function generate_time_variable! end
 function generate_internal_model end
 function generate_state_variable! end
 function generate_input_variable! end
+function generate_tunable_params! end
 function generate_timescale! end
 function add_initial_constraints! end
 function add_constraint! end
+# Default: return P[i] directly. Symbolic backends (like Pyomo) can override this.
+get_param_for_pmap(model, P, i) = P isa AbstractArray ? P[i] : P
+# Some backends need symbolic accessors instead of raw variables (CasADi in particular)
+needs_individual_tunables(model) = false
+
+function f_wrapper(f, Uₙ, Vₙ, p, P, t)
+    if isempty(P)
+        # no tunable parameters
+        return f(Uₙ, Vₙ, p, t)
+    end
+    if SciMLStructures.isscimlstructure(p)
+        _, repack, _ = SciMLStructures.canonicalize(SciMLStructures.Tunable(), p)
+        p′ = repack(P)
+        f(Uₙ, Vₙ, p′, t)
+    else
+        f(Uₙ, Vₙ, P, t)
+    end
+end
 
 function set_variable_bounds!(m, sys, pmap, tf)
     @unpack model, U, V, tₛ = m
@@ -443,8 +528,8 @@ function substitute_toterm(vars, exprs)
     exprs = map(c -> Symbolics.fast_substitute(c, toterm_map), exprs)
 end
 
-function substitute_params(pmap, exprs)
-    exprs = map(c -> Symbolics.fixpoint_sub(c, Dict(pmap)), exprs)
+function substitute_params(pmap::Dict, exprs)
+    exprs = map(c -> Symbolics.fixpoint_sub(c, pmap), exprs)
 end
 
 function check_constraint_vars(vars)
@@ -467,6 +552,7 @@ function prepare_and_optimize! end
 function get_t_values end
 function get_U_values end
 function get_V_values end
+function get_P_values end
 function successful_solve end
 
 """
@@ -474,17 +560,25 @@ function successful_solve end
 
 - kwargs are used for other options. For example, the `plugin_options` and `solver_options` will propagated to the Opti object in CasADi.
 """
-function DiffEqBase.solve(prob::AbstractDynamicOptProblem,
+function CommonSolve.solve(prob::SciMLBase.AbstractDynamicOptProblem,
         solver::AbstractCollocation; verbose = false, kwargs...)
     solved_model = prepare_and_optimize!(prob, solver; verbose, kwargs...)
 
     ts = get_t_values(solved_model)
     Us = get_U_values(solved_model)
     Vs = get_V_values(solved_model)
+    Ps = get_P_values(solved_model)
     is_free_final(prob.wrapped_model) && (ts .+ prob.tspan[1])
 
-    ode_sol = DiffEqBase.build_solution(prob, solver, ts, Us)
-    input_sol = isnothing(Vs) ? nothing : DiffEqBase.build_solution(prob, solver, ts, Vs)
+    # update the parameters with the ones in the solved_model
+    if !isempty(Ps)
+        new_p = SciMLStructures.replace(SciMLStructures.Tunable(), prob.p, Ps)
+        new_prob = remake(prob, p=new_p)
+    else
+        new_prob = prob
+    end
+    ode_sol = SciMLBase.build_solution(new_prob, solver, ts, Us)
+    input_sol = isnothing(Vs) ? nothing : SciMLBase.build_solution(new_prob, solver, ts, Vs)
 
     if !successful_solve(solved_model)
         ode_sol = SciMLBase.solution_new_retcode(
